﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Linq;

namespace HeuristicLab.Services.Hive.DataAccess.Daos.HiveStatistics {
  public class FactTaskDao : GenericDao<Guid, FactTask> {
    private const string BatchDeleteQuery =
      @"DELETE FROM [statistics].[FactTask]
         WHERE TaskId IN ({0});";
    public FactTaskDao(DataContext dataContext) : base(dataContext) { }

    public override FactTask GetById(Guid id) {
      return Table.SingleOrDefault(x => x.TaskId == id);
    }

    public IQueryable<FactTask> GetNotFinishedTasks() {
      return Table.Where(x => x.EndTime == null);
    }

    public IQueryable<FactTask> GetByJobId(Guid id) {
      return Table.Where(x => x.JobId == id);
    }

    public override void Delete(IEnumerable<Guid> ids) {
      string paramIds = string.Join(",", ids.Select(x => string.Format("'{0}'", x)));
      if (!string.IsNullOrWhiteSpace(paramIds)) {
        string query = string.Format(BatchDeleteQuery, paramIds);
        DataContext.ExecuteCommand(query);
      }
    }

  }
}
